// DataBuffer.cpp: implementation of the CDataBuffer class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DataBuffer.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDataBuffer::CDataBuffer()
{
	m_dwSize = 0;
	m_dwPos = 0;
	m_pBuffer = NULL;
}

CDataBuffer::~CDataBuffer()
{
	if( m_pBuffer != NULL )
		delete[] m_pBuffer;
}

/*///////////////////////////////////////////////////////////////
 /                           Allocate                           /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Allocate space for buffer of size dwSize      /
 / Parameters  :  dwSize - size of buffer needed                /
 / Return Value:  bool - was buffer successfully allocated?     /
 / Remarks     :  deallocated buffer in necessary.              /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
bool CDataBuffer::Allocate(DWORD dwSize)
{
	/// can't allocate less than 1 byte
	assert( dwSize > 0 );

	/// deallocate if buffer exists
	if( m_pBuffer != NULL )
		delete[] m_pBuffer;

	m_dwSize = dwSize;
	m_dwPos = 0; /// reset read cursor

	/// allocate buffer
	m_pBuffer = new BYTE[m_dwSize];
	return m_pBuffer != NULL;
}

/*///////////////////////////////////////////////////////////////
 /                            Append                            /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Write data to buffer at current cursor pos.   /
 / Parameters  :  pData     - data to write to buffer           /
 /                nDataSize - size of data                      /
 / Return Value:  bool - was data sucessfully written?          /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
bool CDataBuffer::Append(BYTE *pData, size_t nDataSize)
{
	assert( pData != NULL );

	/// overflow
	if( nDataSize + m_dwPos > m_dwSize )
		return false;

	/// copy into buffer
	memcpy( m_pBuffer + m_dwPos, pData, nDataSize );
	/// advance cursor
	m_dwPos += nDataSize;
	return true;
}


/*///////////////////////////////////////////////////////////////
 /                             Size                             /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Get the size of the buffer                    /
 / Parameters  :  None                                          /
 / Return Value:  DWORD - buffer size                           /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
DWORD CDataBuffer::Size() const
{
	return m_dwSize;
}

/*///////////////////////////////////////////////////////////////
 /                            ToChar                            /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Converts buffer from BYTE to Char             /
 / Parameters  :  szData - buffer to receive the char           /
 /                nSize  - size of szData                       /
 / Return Value:  None                                          /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
void CDataBuffer::ToChar(char *szData, size_t nSize)
{
	assert( szData != NULL );
	assert( m_pBuffer != NULL );

	ByteToChar( m_pBuffer, szData, nSize );
}

/*///////////////////////////////////////////////////////////////
 /                           GetLine                            /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Get a single line of character data from the  /
 /					buffer.										/
 / Parameters  :  szData - memory to receive the line of data   /
 /                nSize  - size of szData                       /
 / Return Value:  bool - true if there any lines left in buffer /
 / Remarks     :  does not return \r\n, will not return a		/
 /					complete line if the line is longer than nSize /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
bool CDataBuffer::GetLine(char *szData, size_t nSize)
{
	assert( szData != NULL );
	assert( nSize > 0 );

	DWORD nLineEnd = memfind( m_pBuffer + m_dwPos, (BYTE*)"\r\n", m_dwSize - m_dwPos, 2 );

	/// \r\n was not found
	if( nLineEnd == -1 )
		return false;

	memset( szData, 0, nSize );
	memcpy( szData, m_pBuffer + m_dwPos, nLineEnd );
	m_dwPos += nLineEnd + 2; // advance 2 more for \r\n
	return true;
}

/*///////////////////////////////////////////////////////////////
 /                           memfind                            /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Finds an array of bytes inside another array  /
 /					of bytes.									/
 / Parameters  :  pSource     - Memory to search in             /
 /                pSub        - Memory to search for            /
 /                nSourceSize - size of pSource                 /
 /                nSubSize    - size of pSub                    /
 / Return Value:  DWORD - the offset in pSource where pSub was  /
 /					found, or -1 if it was not found.			/
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
DWORD CDataBuffer::memfind(const BYTE *pSource, const BYTE *pSub, 
														size_t nSourceSize, size_t nSubSize)
{
	assert( pSource != NULL );
	assert( pSub != NULL );

	for( long i = 0; i < nSourceSize - nSubSize;i++ )
		if( memcmp( pSource+i, pSub, nSubSize ) == 0 )
			return i;
	return -1;
}

/*///////////////////////////////////////////////////////////////
 /                          CharToByte                          /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Converts an array of chars to an array of BYTE/
 / Parameters  :  pChar - pointer to character array            /
 /                pByte - pointer to byte array                 /
 /                count - size of pChar                         /
 / Return Value:  None                                          /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
void CDataBuffer::CharToByte(const char *pChar, BYTE *pByte, size_t count)
{
	assert( pByte != NULL );
	assert( pChar != NULL );
	for(int i = 0; i < count;i++ )
		pByte[i] = (byte)pChar[i];
}

/*///////////////////////////////////////////////////////////////
 /                          ByteToChar                          /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Converts an array of BYTES to chars           /
 / Parameters  :  pByte - array of bytes (source)               /
 /                pChar - array of chars (dest)                 /
 /                count - size of pByte                         /
 / Return Value:  None                                          /
 / Remarks     :  pChar must be big enough for null terminator  /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
void CDataBuffer::ByteToChar(const BYTE *pByte, char *pChar, size_t count)
{
	/// we must correct for signed bit
	for(int i = 0; i < count;i++ )
		pChar[i] = ((char)pByte[i]) & 0xff;
	pChar[count] = 0;
}

/*///////////////////////////////////////////////////////////////
 /                           IsEmpty                            /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Is buffer empty?                              /
 / Parameters  :  None                                          /
 / Return Value:  bool                                          /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
bool CDataBuffer::IsEmpty() const
{
	return m_pBuffer == NULL;
}

/*///////////////////////////////////////////////////////////////
 /                             Seek                             /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Seeks to new position in the buffer           /
 / Parameters  :  dwPos - position to seek to.                  /
 / Return Value:  None                                          /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
void CDataBuffer::Seek(DWORD dwPos)
{
	assert( dwPos >= 0 );
	assert( dwPos < m_dwSize );

	m_dwPos = dwPos;
}

/*///////////////////////////////////////////////////////////////
 /                            GetPos                            /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Gets current position in the buffer           /
 / Parameters  :  None                                          /
 / Return Value:  DWORD                                         /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
DWORD CDataBuffer::GetPos() const
{
	return m_dwPos;
}

/*///////////////////////////////////////////////////////////////
 /                           FindNext                           /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Finds next occurence of szSub in the buffer.  /
 / Parameters  :  szSub - String to search for                  /
 /                nPos  - position to search at, default = 0    /
 / Return Value:  DWORD - position szSub was found at, or -1    /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
DWORD CDataBuffer::FindNext(const char *szSub, DWORD dwPos)
{
	assert( szSub != NULL );
	assert( dwPos >= 0 );
	assert( dwPos < m_dwSize );

	/// convert to byte array
	DWORD nSize = strlen(szSub);
	BYTE* pbSub = new BYTE[nSize];
	CharToByte( szSub, pbSub, nSize );

	DWORD dwFound = memfind( m_pBuffer + dwPos, pbSub, m_dwSize - dwPos, nSize );
	delete[] pbSub;
	return dwFound;
}

/*///////////////////////////////////////////////////////////////
 /                             Copy                             /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Copies bytes the buffer into pData			/
 / Parameters  :  pData - buffer to receive data                /
 /                nSize - size of pData                         /
 / Return Value:  bool - returns false if nSize is greater than /
 /					space left in buffer.						/
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
bool CDataBuffer::Copy(BYTE *pData, size_t nSize)
{
	assert( nSize > 0 );
	assert( pData != NULL );

	/// trying to copy more data than available
	if( m_dwPos + nSize > m_dwSize )
		return false;

	memcpy( pData, m_pBuffer + m_dwPos, nSize );
	m_dwPos += nSize;
	return true;
}

/*///////////////////////////////////////////////////////////////
 /                             Copy                             /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Copies bytes the buffer into szData			/
 / Parameters  :  szData - buffer to receive data               /
 /                nSize - size of pData                         /
 / Return Value:  bool - returns false if nSize is greater than /
 /					space left in buffer.						/
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
bool CDataBuffer::Copy(char *szData, size_t nSize)
{
	assert( szData != NULL );
	assert( nSize > 0 );

	BYTE* pData = new BYTE[nSize];
	bool result = Copy(pData,nSize);
	ByteToChar(pData,szData,nSize);
	delete[] pData;
	return result;
}
